/* Minimum Cut Linear Arrangement */

/* Written by Qiong Cheng <qcheng1@student.gsu.edu> */

/* The Minimum Cut Linear Arrangement is stated as follows.
   Let a directed graph G = (V, E) be given, where V = {1, ..., n} is
   a set of nodes, E <= V x V is a set of arcs. Let also each arc
   e = (i,j) be assigned a number w[i,j], which is the weight of the
   arc e. The problem is to find a permutation which meets the 
   minimum of maximum number of cut edges in any integer point. */

param n, integer, >= 3;
/* number of nodes */

param m, integer, >= 3;
/* number of edges */

set V := 0..n-1;
/* set of nodes */

set E, within V cross V;
/* set of arcs */

param W{(i,j) in E};
/* weight from node i to node j */

var X{i in 0..n-1, j in 1..n}, integer, >= 0, <= 1;
/* X represents as the one-one function; for any i larger than 1 and less than n, 
   a list of reduced choice is a permutation;
   X[i,j,t] = 1 means that the node i is reduced to j */

var c{i in 0..n-1, j in 0..n-1, k in 1..n}, integer, >= 0, <= 1;

var Z, integer,>=0, <= m;

minimize MinCut: Z;
/* the objective is to find a permutation which meets the minimum number of
   maximum number of cut edges in any integer point. */

s.t. oneToConstraint{j in 1..n}: (sum{i in 0..n-1} X[i,j]) == 1;
/* constraint: one-to-one function 1 */

s.t. toOneConstraint{i in 0..n-1}: (sum{j in 1..n} X[i,j]) == 1;
/* constraint: one-to-one function 2 */

#s.t. reductionConstraint{k in 1..n, u in 1..n, v in 1..n, (i,j) in E}: if (u<=k and v>k and W[i,j]>=1) then X[i,u]+X[j,v]-1 <= c[i,j,k];
s.t. reductionConstraint{k in 1..n, u in 1..n, v in 1..n, (i,j) in E : u<=k and v>k and W[i,j]>=1}: X[i,u]+X[j,v]-1 <= c[i,j,k];
#s.t. f2Constraint{u in 1..n, v in 1..n, k in 1..n, (i,j) in E}: k*W[i,j]*c[i,j,k], >= sum{j in 1..n} v*W[i,j]*X[j,v], < sum{i in 1..n} u*W[i,j]*X[i,u];
s.t. maxConstraint{k in 1..n}: (sum{i in 0..n-1, j in 0..n-1} W[i,j]*c[i,j,k]) <= Z;

#s.t. f1Constraint{u in 1..n, v in 1..n, k in 1..n}: sum{i in 1..n} u*X[i,u] <= k*W[i,j]*c[i,j,k];
#s.t. f2Constraint{u in 1..n, v in 1..n, k in 1..n}: sum{j in 1..n} v*X[j,v] > k*W[i,j]*c[i,j,k];

solve;

printf "Result : Min Cut Linear Arrangement = %d\n", Z;

#printf("one-to-one function :\n");
#printf{i in 1..n, j in 1..n} "%d - %d - %d\n", i, j, X[i,j];

end;

