/* Minimum Cut Linear Arrangement */

/* Written by Qiong Cheng <qcheng1@student.gsu.edu> */

/* The Minimum Cut Linear Arrangement is stated as follows.
   Let a directed graph G = (V, E) be given, where V = {1, ..., n} is
   a set of nodes, E <= V x V is a set of arcs. Let also each arc
   e = (i,j) be assigned a number w[i,j], which is the weight of the
   arc e. The problem is to find a permutation which meets the 
   minimum of maximum number of cut edges in any integer point. */

param n, integer, >= 3;
/* number of nodes */

param m, integer, >= 3;
/* number of edges */

set V := 0..n-1;
/* set of nodes */

set E, within V cross V;
/* set of arcs */

param W{(i,j) in E};
/* weight from node i to node j */

var M, integer, <= m;
/* maximum number of cut edges in any integer point (t) */

var X{i in 1..n, k in 1..n}, integer, >= 0, <= 1;
/* X represents as the one-one function; for any i larger than 1 and less than n, 
   a list of reduced choice is a permutation;
   X[i,k] = 1 means that the node i is reduced to k */

var e{i in 1..n, k in 1..n}, integer, <= n;
/* e[i,k] : to estimate the contribution of vertex i to a 
   a cut edge between intervals F(k-1) and F(j) assuming F(i)<k-1; */

var q, >= 0;
/* random real number q */

minimize MinCut: M;
/* the objective is to find a permutation which meets the minimum number of
   maximum number of cut edges in any integer point. */

s.t. oneToConstraint{k in 1..n}: (sum{i in 1..n} X[i,k]) == 1;
/* constraint: one-to-one function 1 */

s.t. toOneConstraint{i in 1..n}: (sum{k in 1..n} X[i,k]) == 1;
/* constraint: one-to-one function 2 */

#s.t. maxCutEdges{k in 1..n}: (sum{i in 1..n : (sum{j in 1..k-1} X[i,j])=1} e[i,k]), <= M;
/* constraint: For any edge(i,j), it meets F(i)<=t<=F(j) *(t-F[j])*/

var P{k in 1..n, i in 1..n};
var R{k in 1..n, i in 1..n};
s.t. tempMediate{k in 1..n, i in 1..n}: R[k,i], = log(P[k,i]);
s.t. tempMediate{k in 1..n, i in 1..n}: R[k,i],  = log(e[i,k]) + log(sum{j in 1..k-1 } X[i,j]);

s.t. maxCutEdges{k in 1..n}: (sum{i in 1..n} P[k,i]), <= M;
/* constraint: For any edge(i,j), it meets F(i)<=t<=F(j) *(t-F[j])*/

s.t. edgeEstimation1{i in 1..n, k in 1..n}: (sum{(i,j) in E, l in k..n} X[j,l]) , <= e[i,k]+ q*n;

s.t. edgeEstimation2{i in 1..n, k in 1..n}: (sum{(i,j) in E, l in k..n} X[j,l]) , >=e[i,k]-q*n;

solve;

end;

