/* Minimum Cut Linear Arrangement */

/* Written by Qiong Cheng <qcheng1@student.gsu.edu> */

/* The Minimum Cut Linear Arrangement is stated as follows.
   Let a directed graph G = (V, E) be given, where V = {1, ..., n} is
   a set of nodes, E <= V x V is a set of arcs. Let also each arc
   e = (i,j) be assigned a number w[i,j], which is the weight of the
   arc e. The problem is to find a permutation which meets the 
   minimum of maximum number of cut edges in any integer point. */

param n, integer, >= 3;
/* number of nodes */

param m, integer, >= 3;
/* number of edges */

set V := 0..n-1;
/* set of nodes */

set E, within V cross V;
/* set of arcs */

param W{(i,j) in E};
/* weight from node i to node j */

var M, integer,>=0, <= m;
/* maximum number of cut edges in any integer point (t) */

var X{i in 0..n-1, j in 0..n-1}, integer, >= 0, <= 1;
/* X represents as the one-one function; for any i larger than 1 and less than n, 
   a list of reduced choice is a permutation;
   X[i,j] = 1 means that the node i is reduced to j */

var e{(i,j) in E, k in 0..n-1}, integer,  >= 0, <= 1;
/* e[i,j,k] :  */

minimize MinCut: M;
/* the objective is to find a permutation which meets the minimum number of
   maximum number of cut edges in any integer point. */

s.t. oneToConstraint{j in 0..n-1}: (sum{i in 0..n-1} X[i,j]) == 1;
/* constraint: one-to-one function 1 */

s.t. toOneConstraint{i in 0..n-1}: (sum{j in 0..n-1} X[i,j]) == 1;
/* constraint: one-to-one function 2 */

s.t. reductionConstraint{k in 0..n-1,(i,j) in E, u in 0..n-1, v in 0..n-1: k < v and u <= k}: (X[i,u]+X[j,v]-1) <= e[i,j,k];

s.t. maxConstraint{k in 0..n-1}: (sum{(i,j) in E} e[i,j,k]) <= M;
 
#s.t. f1Constraint{k in 0..n-1, (i,j) in E}: (sum{u in 0..n-1} u*X[i,u]) <= k*e[i,j,k];

#s.t. f2Constraint{k in 0..n-1, (i,j) in E}:   k*e[i,j,k] <= (sum{v in 0..n-1} v*X[j,v]);

solve;

printf "Result : Min Cut Linear Arrangement = %d\n", M;
printf("one-to-one function :\n");
printf{i in 0..n-1, j in 0..n-1} "%d - %d - %d\n", i, j, X[i,j];
end;

