/* Minimum Cut Linear Arrangement */

/* Written by Qiong Cheng <qcheng1@student.gsu.edu> */

/* The Minimum Cut Linear Arrangement is stated as follows.
   Let a directed graph G = (V, E) be given, where V = {1, ..., n} is
   a set of nodes, E <= V x V is a set of arcs. Let also each arc
   e = (i,j) be assigned a number w[i,j], which is the weight of the
   arc e. The problem is to find a permutation which meets the 
   minimum of maximum number of cut edges in any integer point. */

param n, integer, >= 3;
/* number of nodes */

param m, integer, >= 3;
/* number of edges */

set V := 0..n-1;
/* set of nodes */

set E, within V cross V;
/* set of arcs */

param W{(i,j) in E};
/* weight from node i to node j */

param T :=(prod{i in 1 ..n} i);

var M{t in 1..T}, integer,>=0, <= m;
/* maximum number of cut edges in any integer point (t) */

var X{i in 1..n, j in 1..n, t in 1..T}, integer, >= 0, <= 1;
/* X represents as the one-one function; for any i larger than 1 and less than n, 
   a list of reduced choice is a permutation;
   X[i,j,t] = 1 means that the node i is reduced to j */

var e{i in 1..n, j in 1..n, k in 1..n, t in 1..T}, integer,  >= 0, <= 1;
/* e[i,j,k,t] :  */

var R, integer,>=0, <= m;

minimize MinCut: R;
/* the objective is to find a permutation which meets the minimum number of
   maximum number of cut edges in any integer point. */

s.t. oneToConstraint{t in 1..T,j in 1..n}: (sum{i in 1..n} X[i,j,t]) == 1;
/* constraint: one-to-one function 1 */

s.t. toOneConstraint{t in 1..T,i in 1..n}: (sum{j in 1..n} X[i,j,t]) == 1;
/* constraint: one-to-one function 2 */

s.t. permutationIConstraint{i in 1..n}: (sum{t in 1..T,j in 1..n} X[i,j,t]) == n;
/* constraint: permutation 1 */

s.t. permutationJConstraint{j in 1..n}: (sum{t in 1..T,i in 1..n} X[i,j,t]) == n;
/* constraint: permutation 2 */

s.t. reductionConstraint{t in 1..T, k in 1..n, u in 1..n, v in 1..n, i in 1..n, j in 1..n: (u,v) in E}: if (i<=k and j>k and W[u,v]>=1) then (X[u,i,t]+X[v,j,t]-1) <= e[i,j,k,t];

s.t. maxConstraint{t in 1..T, k in 1..n}: (sum{i in 1..n, j in 1..n} e[i,j,k, t]) <= M[t];

s.t. minCutConstraint{t in 1..T}: M[t]>=R;

solve;

printf "Result : Min Cut Linear Arrangement = %d\n", R;

#printf("one-to-one function :\n");
#printf{i in 1..n, j in 1..n} "%d - %d - %d\n", i, j, X[i,j];

end;

