#include "MinCutLinearArrangement.h"
/* Written by Qiong Cheng <qcheng1@student.gsu.edu> */

/* The Minimum Cut Linear Arrangement is stated as follows.
   Let a directed graph G = (V, E) be given, where V = {1, ..., n} is
   a set of nodes, E <= V x V is a set of arcs. Let also each arc
   e = (i,j) be assigned a number w[i,j], which is the weight of the
   arc e. The problem is to find a permutation which meets the 
   minimum of maximum number of cut edges in any integer point. */

void getInputParameter(char *, char *);
void InitializeGraph(char*, int *, int *);
void InitializeInputArray(char *, int **, int);

int num_of_vertices, num_of_edges, **input_arr;

int main(int argc, char *argv[]){	
	int i = 0, j = 0,count=1;
    LPX *lp;
    char *name;
    int k=0,index=0,u,v,t;
	int rowNum = 0;   
    int numVariables = 0;
    int *ia, *ja;
    double *ar, res;
        	
	if ( argc < 1 ){
		printf("MinCutLA filename\n");
		return 0;
	}
	
	printf("argc = %d\n", argc);
	for(i = 0; i < argc; i++)
		printf("arg %d: %s\n", i, argv[i]);

	/* Initialization Phase...*/
	InitializeGraph(argv[1], &num_of_vertices, &num_of_edges);
	
	input_arr = (int **)malloc(sizeof(int)*num_of_edges);
	for (i=0;i<num_of_edges;i++){
		input_arr[i] = (int *)malloc(sizeof(int)*3);
	}
	
	InitializeInputArray(argv[1], input_arr,num_of_vertices);
	
	/* Check the input file...*/
	if (isDebug == 1) printf("%d %d\n", num_of_vertices, num_of_edges);	
	for (i=0;i<num_of_edges;i++){
		if (isDebug == 1) printf("%d %d %d \n",input_arr[i][0], input_arr[i][1],input_arr[i][2]);
	}
	
	/*greedy heuristic approach...*/ 
	rowNum = num_of_vertices + num_of_vertices + num_of_edges*num_of_vertices + num_of_edges*num_of_vertices*num_of_vertices*num_of_vertices;   
    numVariables = 1 + num_of_vertices*num_of_vertices + num_of_vertices*num_of_edges;
    ia = (int *)malloc(sizeof(int)*(1+rowNum*numVariables));
    ja = (int *)malloc(sizeof(int)*(1+rowNum*numVariables));
    ar = (double *)malloc(sizeof(double)*(1+rowNum*numVariables));
    name = (char *)malloc(sizeof(char)*(MAX_STR_LENGTH));
	  
    lp = lpx_create_prob();
    lpx_set_prob_name(lp, "MinCutLA");
	lpx_set_class(lp, LPX_MIP);    
    lpx_set_obj_dir(lp, LPX_MIN);
    	
    lpx_add_cols(lp, numVariables+1);
    for (i =0; i< numVariables; i++){
    	if (i==0){/* M */
    		lpx_set_col_kind(lp, i+1, LPX_IV);
		    lpx_set_col_bnds(lp, i+1, LPX_DB, 0.0, num_of_edges);
		    lpx_set_obj_coef(lp, i+1, 1);
    	}else if (i <= num_of_vertices*num_of_vertices ){/* X[0..n-1, 0..n-1] */
    		lpx_set_col_kind(lp, i+1, LPX_IV);    		
		    lpx_set_col_bnds(lp, i+1, LPX_DB, 0.0, 1.0);
		    lpx_set_obj_coef(lp, i+1, 0);
    	}else{/* e[edge, 0..n-1] */
    		lpx_set_col_kind(lp, i+1, LPX_IV);    		
		    lpx_set_col_bnds(lp, i+1, LPX_DB, 0.0, 1.0);
		    lpx_set_obj_coef(lp, i+1, 0);
    	}
    }
    if (isDebug == 1) printf("%d, %d %d \n", rowNum, numVariables, 1+rowNum*numVariables);
        
    lpx_add_rows(lp, rowNum+1);   
    for (j =0; j < 2*num_of_vertices+ num_of_edges*num_of_vertices; j++){
    	/* s.t. oneToConstraint{j in 0..n-1}: (sum{i in 0..n-1} X[i,j]) == 1; */
	    if (j < num_of_vertices){
	    	lpx_set_row_bnds(lp, j+1, LPX_FX, 1.0, 1.0);
	    	
	    	/*Coefficiency */
	    	k = j;
	    	for (i =1; i <= num_of_vertices*num_of_vertices; i++){
	    		if ((i-1)%num_of_vertices == k){
	    			ia[count] = j+1;
	    			ja[count] = i+1;
	    			
	    			ar[count++] = 1;
	    		}
	    	}
	    }else if (j< 2*num_of_vertices){
	    	/* s.t. toOneConstraint{i in 0..n-1}: (sum{j in 0..n-1} X[i,j]) == 1; */
    		lpx_set_row_bnds(lp, j+1, LPX_FX, 1.0, 1.0);
    		
    		/*Coefficiency */
	    	k = j-num_of_vertices;
	    	for (i =1; i <=num_of_vertices*num_of_vertices; i++){
	    		if ((i-1)/num_of_vertices == k){
	    			ia[count] = j+1;
	    			ja[count] = i+1;
	    			ar[count++] = 1;
	    		}
	    	}
	    }else if (j<2*num_of_vertices+ num_of_edges*num_of_vertices){
	    	/*s.t. maxConstraint{k in 0..n-1}: (sum{(i,j) in E} e[i,j,k]) <= M;*/
    		lpx_set_row_bnds(lp, j+1, LPX_LO, 0.0, 0.0);
    		ia[count] = j+1;
			ja[count] = 1;
			ar[count++] = 1;
			
    		/*Coefficiency */
	    	k = j-2*num_of_vertices;
	    	for (i =num_of_vertices*num_of_vertices+1; i < numVariables; i++){
	    		if ((i-1-num_of_vertices*num_of_vertices)%num_of_vertices == k){
	    			ia[count] = j+1;
	    			ja[count] = i+1;
	    			ar[count++] = -1;
	    		}
	    	}
	    }
    }

	{
    	/*Coefficiency */
    	/* s.t. reductionConstraint{k in 0..n-1,(i,j) in E, u in 0..n-1, v in 0..n-1: k < v and u <= k}: (X[i,u]+X[j,v]-1) <= e[i,j,k];
    	 */
    	for (index =0; index < num_of_edges; index++){
    		i = input_arr[index][0];
    		t = input_arr[index][1];
			for (u=0;u< num_of_vertices;u++){
				for (v=0;v< num_of_vertices;v++){
					for (k=0;k< num_of_vertices;k++){
						if (k<v && u<=k){
    						lpx_set_row_bnds(lp, j+1, LPX_LO, -1.0, 1.0);								
							ia[count] = j+1;
				    		ja[count] = i*num_of_vertices + u+2;
							ar[count++] = -1;
							
 							ia[count] = j+1;
				    		ja[count] = t*num_of_vertices + v+2;								
							ar[count++] = -1;							
							
							ia[count] = j+1;
				    		ja[count] = index*num_of_vertices + k+1+num_of_vertices*num_of_vertices+1;									
							ar[count++] = 1;
							j++;
						}
					}   		
				}   		
			}   		
    	}
    }
	if (isDebug == 1) printf("%d\n", count);
	
    lpx_load_matrix(lp, count-1, ia, ja, ar);
    lpx_simplex(lp);
    printf("\nZ = %d ;\n", lpx_mip_obj_val(lp) );
    
	for (u=0;u< num_of_vertices;u++){
		for (v=0;v< num_of_vertices;v++){
			res = lpx_mip_col_val(lp, 1+u*num_of_vertices +v);
			if (res != 0)
				printf("%d corresponds to %d, isBelongTo=%e\n", u, v, res);
		}   		
	} 
	for (u=0;u< num_of_edges;u++){
		for (v=0;v< num_of_vertices;v++){
			res = lpx_mip_col_val(lp, 1+num_of_vertices*num_of_vertices+u*num_of_vertices +v);
			if (res !=0)
				printf("edge from=%d to=%d k=%d, isBelongTo=%e\n", input_arr[u][0],input_arr[u][1], v, res);
		}   		
	}     		
    lpx_delete_prob(lp);
    return 0;
}

void InitializeGraph(char *inputF, int *num_of_vertices, int *num_of_edges){
	FILE *f_input_file;
	char line[MAX_STR_LENGTH],ch[16];

	f_input_file = fopen(inputF, "r");
	if (f_input_file == NULL){
		printf("Cannot open input file!");
		exit(1);
	}

	/* Getting the number of vertices, and number of edges parameters for the input graph */
	fgets(line,MAX_STR_LENGTH,f_input_file);
	getInputParameter(line, ch );
	*num_of_vertices = atoi(ch);
	
	fgets(line,MAX_STR_LENGTH,f_input_file);
	getInputParameter(line, ch );
	*num_of_edges = atoi(ch);
		
	fclose(f_input_file);
}

void getInputParameter(char * line, char * ch ){
	int isFirst = 1,i=0,j=0;
	if (line == NULL || ch == NULL)
		return;
		
	while (line[i]!='\0'){
		if ( isFirst && line[i] != '=' )
			i++;
		else {
			if ( isFirst == 1 ){
				isFirst = 0;
				i++;
			}
			
			ch[j++] = line[i++];
		}

	}
	ch[j]='\0';
}

void InitializeInputArray(char * inputF, int **input_arr, int num_of_vertices){
	FILE *f_input_file;
	char line[MAX_STR_LENGTH],ch[16];
	int graph_parameters[3],i,j,k=0,index=0;

	for (i=0;i<num_of_vertices;i++){
		for (j=0;j<num_of_vertices;j++){
			input_arr[i][j]=0;
		}
	}

	f_input_file = fopen(inputF, "r");
	if (f_input_file == NULL){
		printf("Cannot open input file!");
		exit(1);
	}
		
	fgets(line,MAX_STR_LENGTH,f_input_file); /* Bypass the first line with number of vertices, edges...etc */
	fgets(line,MAX_STR_LENGTH,f_input_file); /* Bypass the first line with number of vertices, edges...etc */

	while (fgets(line,MAX_STR_LENGTH,f_input_file)){
		k = 0;
		i = 0;
		j = 0;
		while (line[i]!='\0'){
			if (line[i]==' '){
				ch[j]='\0';	
				graph_parameters[k++] = atoi(ch);
				j=0;
				i++;
				continue;
			}
	
			ch[j++] = line[i++];
		}
	
		ch[j]='\0';
		graph_parameters[k] = atoi(ch);
		
		input_arr[index][0] =graph_parameters[0];
		input_arr[index][1] =graph_parameters[1];
		if (k>=2)
			input_arr[index][2] =graph_parameters[2];
		else
			input_arr[index][2] = 1;
		index++;
	}

	fclose(f_input_file);
}
