#include "MinCutLinearArrangement.h"

void getInputParameter(char *, char *);
void InitializeGraph(char*, int *, int *);
void InitializeInputArray(char *, int **, int);

int num_of_vertices, num_of_edges, **input_arr;

int main(int argc, char *argv[]){	
	int i = 0, j = 0;
	if ( argc < 1 ){
		printf("MinCutLA filename\n");
		return 0;
	}
	
	printf("argc = %d\n", argc);
	for(i = 0; i < argc; i++)
		printf("arg %d: %s\n", i, argv[i]);

	/* Initialization Phase...*/
	InitializeGraph(argv[1], &num_of_vertices, &num_of_edges);
	
	input_arr = (int **)malloc(sizeof(int)*num_of_vertices);
	for (i=0;i<num_of_vertices;i++){
		input_arr[i] = (int *)malloc(sizeof(int)*num_of_vertices);
	}
	
	InitializeInputArray(argv[1], input_arr,num_of_vertices);
	
	/* Check the input file...*/
	for (i=0;i<num_of_vertices;i++){
		for (j=0;j<num_of_vertices;j++){
			if (isDebug == 1) printf("%d ",input_arr[i][j]);
		}
		if (isDebug == 1) printf("\n");
	}
	
	/*greedy heuristic approach...*/
    LPX *lp;
    int ia[1+1000], ja[1+1000];
    double ar[1+1000], Z, x1, x2, x3;	
    lp = lpx_create_prob();
    lpx_set_prob_name(lp, "sample");
    lpx_set_obj_dir(lp, LPX_MAX);
    lpx_add_rows(lp, 3);
    lpx_set_row_name(lp, 1, "p");
    lpx_set_row_bnds(lp, 1, LPX_UP, 0.0, 100.0);
    lpx_set_row_name(lp, 2, "q");
    lpx_set_row_bnds(lp, 2, LPX_UP, 0.0, 600.0);
    lpx_set_row_name(lp, 3, "r");
    lpx_set_row_bnds(lp, 3, LPX_UP, 0.0, 300.0);
    lpx_add_cols(lp, 3);
    lpx_set_col_name(lp, 1, "x1");
    lpx_set_col_bnds(lp, 1, LPX_LO, 0.0, 0.0);
    lpx_set_obj_coef(lp, 1, 10.0);
    lpx_set_col_name(lp, 2, "x2");
    lpx_set_col_bnds(lp, 2, LPX_LO, 0.0, 0.0);
    lpx_set_obj_coef(lp, 2, 6.0);
    lpx_set_col_name(lp, 3, "x3");
    lpx_set_col_bnds(lp, 3, LPX_LO, 0.0, 0.0);
    lpx_set_obj_coef(lp, 3, 4.0);
    ia[1] = 1, ja[1] = 1, ar[1] =  1.0; /* a[1,1] =  1 */
    ia[2] = 1, ja[2] = 2, ar[2] =  1.0; /* a[1,2] =  1 */
    ia[3] = 1, ja[3] = 3, ar[3] =  1.0; /* a[1,3] =  1 */
    ia[4] = 2, ja[4] = 1, ar[4] = 10.0; /* a[2,1] = 10 */
    ia[5] = 3, ja[5] = 1, ar[5] =  2.0; /* a[3,1] =  2 */
    ia[6] = 2, ja[6] = 2, ar[6] =  4.0; /* a[2,2] =  4 */
    ia[7] = 3, ja[7] = 2, ar[7] =  2.0; /* a[3,2] =  2 */
    ia[8] = 2, ja[8] = 3, ar[8] =  5.0; /* a[2,3] =  5 */
    ia[9] = 3, ja[9] = 3, ar[9] =  6.0; /* a[3,3] =  6 */
    lpx_load_matrix(lp, 9, ia, ja, ar);
    lpx_simplex(lp);
    Z = lpx_get_obj_val(lp);
    x1 = lpx_get_col_prim(lp, 1);
    x2 = lpx_get_col_prim(lp, 2);
    x3 = lpx_get_col_prim(lp, 3);
    printf("\nZ = %g; x1 = %g; x2 = %g; x3 = %g\n", Z, x1, x2, x3);
    lpx_delete_prob(lp);
    return 0;
}

void InitializeGraph(char *inputF, int *num_of_vertices, int *num_of_edges){
	FILE *f_input_file;
	char line[MAX_STR_LENGTH],ch[16];

	f_input_file = fopen(inputF, "r");
	if (f_input_file == NULL){
		printf("Cannot open input file!");
		exit(1);
	}

	/* Getting the number of vertices, and number of edges parameters for the input graph */
	fgets(line,MAX_STR_LENGTH,f_input_file);
	getInputParameter(line, ch );
	*num_of_vertices = atoi(ch);
	if (isDebug == 1) printf("%d ", *num_of_vertices);
	
	fgets(line,MAX_STR_LENGTH,f_input_file);
	getInputParameter(line, ch );
	*num_of_edges = atoi(ch);
	if (isDebug == 1) printf("%d ", *num_of_edges);
		
	fclose(f_input_file);
}

void getInputParameter(char * line, char * ch ){
	int isFirst = 1,i=0,j=0;
	if (line == NULL || ch == NULL)
		return;
		
	while (line[i]!='\0'){
		if ( isFirst && line[i] != '=' )
			i++;
		else {
			if ( isFirst == 1 ){
				isFirst = 0;
				i++;
			}
			
			ch[j++] = line[i++];
		}

	}
	ch[j]='\0';
}

void InitializeInputArray(char * inputF, int **input_arr, int num_of_vertices){
	FILE *f_input_file;
	char line[MAX_STR_LENGTH],ch[16];
	int graph_parameters[3],i,j,k=0;

	for (i=0;i<num_of_vertices;i++){
		for (j=0;j<num_of_vertices;j++){
			input_arr[i][j]=0;
		}
	}

	f_input_file = fopen(inputF, "r");
	if (f_input_file == NULL){
		printf("Cannot open input file!");
		exit(1);
	}
		
	fgets(line,MAX_STR_LENGTH,f_input_file); /* Bypass the first line with number of vertices, edges...etc */
	fgets(line,MAX_STR_LENGTH,f_input_file); /* Bypass the first line with number of vertices, edges...etc */

	while (fgets(line,MAX_STR_LENGTH,f_input_file)){
		k = 0;
		i = 0;
		j = 0;
		while (line[i]!='\0'){
			if (line[i]==' '){
				ch[j]='\0';	
				graph_parameters[k++] = atoi(ch);
				j=0;
				i++;
				continue;
			}
	
			ch[j++] = line[i++];
		}
	
		ch[j]='\0';
		graph_parameters[k] = atoi(ch);
		if (k>=2)
			input_arr[graph_parameters[0]][graph_parameters[1]] = graph_parameters[2];
		else
			input_arr[graph_parameters[0]][graph_parameters[1]] = 1;
		
	}

	fclose(f_input_file);
}