#include <iostream>
#include <stdlib.h>
#include <string>
#include <fstream>

extern "C" {
#include "glpk.h"
}

using namespace std;

int main(void){

	string filename;
	string line;
	int n;		// the number of vertices
	int m;		// the number of edges
	int position;	// position index use in string
	int *edgeI;	// pointer to array of node i of edge(i,j) 
	int *edgeJ;	// pointer to array of node j of edge(i,j)
	//int le;	// length index 
	int e;		// edge index
	int v1,v2; 	// vertices index
	int c1, c2;
	int i = 1;	// array index

	cout << "Please enter input file: ";
	cin >> filename;
	
	// open input file
	ifstream inputfile (filename.c_str());
	
	if (inputfile.is_open()){

		// get the first line to extract n 
		getline (inputfile, line);
		position = line.find('=');
		n = atoi(line.substr(position+1, line.length()-position).c_str());
		
		// get the second line to extract m 
		getline (inputfile, line);
		position = line.find('=');
		m = atoi(line.substr(position+1, line.length()-position).c_str());
		
		// allocate memory to store edges
		edgeI = new int[m];
		edgeJ = new int[m];
		
		// read edges
		for (e=0; e<m; e++){
			inputfile >> edgeI[e];
			inputfile >> edgeJ[e];
		}
		
		// close file
		inputfile.close();

	} else{
		cout<< "Unable to open input file";
		return 0;
	}


	// variables: leij 	= length of edge (i,j)
		
	// variables: xi,k	= 1 when node i is assigned to place k
	//			  0 otherwise
	
	int row;	// row index
	int row2;
	int col;	// column index
	

	// objective: min sum (leij)

	// constraint#1: every vertex has only one place
	//		for every i:	sum (xi,k) over k = 1
	
	// constrain#2: every place has only for one vertex
	//		for every k:	sum (xi,k) over i = 1

	// constraint#3: each edge has length > 0
	//		for every edge(i,j):	sum(u(xi,u)) - sum(v(xj,v)) - leij <=0		
	//      for every edge(i,j):	sum(u(xi,u)) - sum(v(xj,v)) - leij >=0
	
	int number_of_rows = 2*n+(2*m);
	// n	 rows	from constraint#1
	// n     rows   from constraint#2
	// 2*m	 rows	for constraint#3
	
	
	int number_of_columns = m+n*n;
	// n*n 	 columns for xi,k variables
	// m     columns for leij variables	

	int matrix_size = number_of_rows* number_of_columns;
	
	// coeficient matrix
	int *ia = new int[1+matrix_size];
	int *ja = new int[1+matrix_size];
      	double *ar = new double[1+matrix_size];

	// create problem and name it
	LPX *lp;
	lp = lpx_create_prob();
	lpx_set_prob_name(lp, "minimum linear arrangement");

	// MIP problem
	lpx_set_class(lp, LPX_MIP);

	// minimize problem
	lpx_set_obj_dir(lp, LPX_MIN);
	
	// add rows to problems
	lpx_add_rows(lp, number_of_rows);
	
	// set bounds to constraint#1 = 1.0
	for ( v1 = 1; v1<=n; v1++){
		row = v1;
		lpx_set_row_bnds(lp, row, LPX_FX, 1.0, 1.0);
	}
	
	// set bounds to constraint#2 = 1.0
	for ( v1 = 1; v1<=n; v1++){
		row = n + v1;
		lpx_set_row_bnds(lp, row, LPX_FX, 1.0, 1.0);
	}
	
	// set bounds to constraint#3 <= 0.0 
	for ( e=1; e<=m; e++){
		row = 2*n+e;
		lpx_set_row_bnds(lp, row, LPX_UP, 0.0, 0.0);
	}
	
	// set bounds to constraint#3 <= 0.0 
	for ( e=1; e<=m; e++){
		row = 2*n+m+e;
		lpx_set_row_bnds(lp, row, LPX_UP, 0.0, 0.0);
	}
	
	// add columns to problems, set their bounds
	lpx_add_cols(lp, number_of_columns);
	
	// xi,k
	for (v1=1; v1<=n; v1++){
		for(c1=1; c1<=n; c1++){
			col =(v1-1)*n+c1;
			lpx_set_col_kind(lp, col, LPX_IV);
			lpx_set_col_bnds(lp, col, LPX_DB, 0.0, 1.0);
			lpx_set_obj_coef(lp, col, 0.0);
		}
	}
	
	// leij
	for (c1 = 1; c1<= m; c1++){
		col = (n*n)+ c1;
		lpx_set_col_kind(lp, col, LPX_IV);
		lpx_set_col_bnds(lp, col, LPX_LO, 1.0, 1.0);
		lpx_set_obj_coef(lp, col, 1.0);
	}
	
	// fill coefficient for constraint#1
	for ( v1 = 1; v1<=n; v1++){
		row = v1;
		// coefficients of all leij are equals to 0
		// coefficient of xi,k over k
		for ( v2 = 1; v2<=n; v2++){
            col = (v1-1)*n+v2;
            ia[i]= row;
		    ja[i]= col;
		    ar[i]= 1;
		    i++;
		}
	}
	
	// fill coefficient for constraint#2
	for ( v1 = 1; v1<=n; v1++){
		row = n+v1;
		// coefficients of all leij are equals to 0
		// coefficient of xi,k over i
		for ( v2 = 1; v2<=n; v2++){
            col = (v2-1)*n+v2;
            ia[i]= row;
		    ja[i]= col;
		    ar[i]= 1;
		    i++;
		}
	}
	

	// fill coefficient for constraint#3
	for (e = 1; e<=m; e++){
        row = 2*n+(2*e)-1;
        row2 = 2*n+(2*e);
        col = (n*n) + e;
        // coefficient of leij
	    ia[i]= row;
	    ja[i]= col;
	    ar[i]= -1;
		cout << " - " << ar[i];
	    i++;
	    ia[i] = row2;
	    ja[i] = col;
	    ar[i] = -1;
		cout << " - " << ar[i];
	    i++;
		// coefficient of xi,k
		for (c1 = 1;c1<=n ; c1++){
            if (c1 == edgeI[e-1]){
		       for(c2=1; c2<=n; c2++){
                   col = (c1-1)*n+c2;
		           ia[i]= row;
		           ja[i]= col;
		           ar[i]= c2;
				   cout << " - " << ar[i];
		           i++;
		           ia[i]= row2;
		           ja[i]= col;
		           ar[i]= -1*c2;
					   cout << " - " << ar[i];
		           i++;
				}
            }
		    if (c1 == edgeJ[e-1]){
               for(c2=1; c2<=n; c2++){
                   col = (c1-1)*n+c2;
		           ia[i]= row;
		           ja[i]= col;
		           ar[i]= -1*c2;
				   cout << " - " << ar[i];
		           i++;
		           ia[i]= row2;
		           ja[i]= col;
		           ar[i]= c2;
				   cout << " - " << ar[i];
		           i++;
               }
			}
		}
		cout << endl;
	}
					
	lpx_load_matrix(lp, --i, ia, ja, ar);
	// use simplex
	lpx_intopt(lp);
	double z = lpx_mip_obj_val(lp);
	cout << "z= " << z << endl;
	// delete problem
	lpx_delete_prob(lp);

	return 0;
}


